"""Top-level package for bq-properties-creator."""

from .datacatalog_util_cli import main  # noqa: F401

__author__ = """Marcelo Miranda"""
__email__ = 'mesmacosta@gmail.com'
__version__ = '0.2.0'

__all__ = ('main')
